@@ -40,13 +40,16 @@
 
 #include <libgnomeprint/gnome-print-config.h>
 #include <libgnomeprint/private/gpa-node.h>
+#include <libgnomeprint/private/gpa-node-private.h>
 #include <libgnomeprint/private/gpa-utils.h>
+#include <libgnomeprint/private/gpa-key.h>
 #include <libgnomeprint/private/gnome-print-private.h>
 #include <libgnomeprint/private/gnome-print-config-private.h>
 
 #include <libgnomeprintui/gnome-print-i18n.h>
 #include <libgnomeprintui/gnome-printer-selector.h>
 #include <libgnomeprintui/gnome-print-content-selector.h>
+#include <libgnomeprintui/gnome-print-layout-selector.h>
 #include <libgnomeprintui/gnome-print-page-selector.h>
 #include <libgnomeprintui/gnome-print-paper-selector.h>
 #include <libgnomeprintui/gnome-print-copies.h>
@@ -61,20 +64,27 @@ enum {
 	PROP_PRINT_CONFIG,
 	PROP_PRINTER_SELECTOR,
 	PROP_CONTENT_SELECTOR,
+	PROP_PAGE_SELECTOR,
 	PROP_NOTEBOOK
 };
 
 static void gnome_print_dialog_class_init (GnomePrintDialogClass *class);
 static void gnome_print_dialog_init (GnomePrintDialog *dialog);
+static void gnome_print_dialog_watch_filter (GnomePrintDialog *, GnomePrintFilter *);
 
 static GtkDialogClass *parent_class;
 
 struct _GnomePrintDialog {
 	GtkDialog dialog;
-	
+
 	GnomePrintConfig *config;
+	GnomePrintFilter *filter;
+
+	GPANode *node_filter, *node_source, *node_printer;
+	guint signal_filter, signal_source, signal_printer;
 
 	GtkWidget *notebook;
+	GtkWidget *l_layout, *l_job, *l_paper;
 
 	/* Containers for selectors */
 	GtkWidget *e_content;
@@ -85,13 +95,13 @@ struct _GnomePrintDialog {
 	GtkWidget *s_page;
 	GtkWidget *s_paper;
 	GtkWidget *s_copies;
-
-	GtkWidget *expert;
+	GtkWidget *s_layout;
 
 	GtkWidget *job;
 	GtkWidget *printer;
 
 	gint flags;
+	gboolean needs_save_filter;
 };
 
 struct _GnomePrintDialogClass {
@@ -118,38 +128,341 @@ gnome_print_dialog_get_type (void)
 }
 
 static void
-gnome_print_dialog_destroy (GtkObject *object)
+on_paper_selector_notify (GObject *object, GParamSpec *pspec,
+		GnomePrintDialog *gpd)
 {
-	GnomePrintDialog *gpd;
-	
-	gpd = GNOME_PRINT_DIALOG (object);
+	GValue v = {0,};
 
-	if (gpd->config) {
-		gpd->config = gnome_print_config_unref (gpd->config);
+	g_value_init (&v, pspec->value_type);
+	if (!strcmp (pspec->name, "width")) {
+		g_object_get_property (object, "width", &v);
+		g_object_set_property (G_OBJECT (gpd->s_layout), "input_width", &v);
+		g_object_set_property (G_OBJECT (gpd->s_layout), "output_width", &v);
 	}
-
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (!strcmp (pspec->name, "height")) {
+		g_object_get_property (object, "height", &v);
+		g_object_set_property (G_OBJECT (gpd->s_layout), "input_height", &v);
+		g_object_set_property (G_OBJECT (gpd->s_layout), "output_height", &v);
+	}
+	g_value_unset (&v);
 }
 
 static void
-gnome_print_dialog_load_config (GnomePrintDialog *gpd)
+on_content_selector_notify (GObject *object, GParamSpec *pspec,
+		GnomePrintDialog *gpd)
 {
-	gint n;
-	GtkWidget *hb, *l;
+	GValue v = {0,};
+
+	g_value_init (&v, pspec->value_type);
+	if (!strcmp (pspec->name, "total") || !strcmp (pspec->name, "current")) {
+		g_object_get_property (object, pspec->name, &v);
+		g_object_set_property (G_OBJECT (gpd->s_page), pspec->name, &v);
+	}
+	g_value_unset (&v);
+}
+
+static GPANode *
+_gnome_print_config_ensure_key (GnomePrintConfig *config, const gchar *key)
+{
+	GPANode *node;
+	const gchar *p;
+	gchar *q;
+
+	g_return_val_if_fail (GNOME_IS_PRINT_CONFIG (config), NULL);
+	g_return_val_if_fail (key, NULL);
+
+	node = gpa_node_lookup (GNOME_PRINT_CONFIG_NODE (config),
+			(const guchar *) key);
+	if (node)
+		return node;
+
+	for (p = key + strlen (key) - 1; (p > key) && (*p != '.'); p--);
+	if (*p == '.') {
+		q = g_strndup (key, p - key);
+		node = gpa_node_lookup (GNOME_PRINT_CONFIG_NODE (config),
+				(const guchar *) q);
+		gpa_key_insert (node, (const guchar *) (p + 1), (const guchar *) "");
+	}
+	return gpa_node_lookup (GNOME_PRINT_CONFIG_NODE (config),
+			(const guchar *) key);
+}
+
+static GtkWidget *
+get_page (GtkWidget *n, GtkWidget *tab)
+{
+	gint i;
+
+	g_return_val_if_fail (GTK_IS_NOTEBOOK (n), NULL);
+	g_return_val_if_fail (GTK_IS_WIDGET (tab), NULL);
+
+	for (i = gtk_notebook_get_n_pages (GTK_NOTEBOOK (n)); i > 0; i--) {
+		GtkWidget *w = gtk_notebook_get_nth_page (GTK_NOTEBOOK (n), i - 1);
+		if (gtk_notebook_get_tab_label (GTK_NOTEBOOK (n), w) == tab)
+			return w;
+	}
+	return NULL;
+}
+
+static void
+gnome_print_dialog_set_has_source (GnomePrintDialog *gpd, gboolean source)
+{
+	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
+
+	gtk_widget_set_sensitive (gpd->l_layout, !source);
+	gtk_widget_set_sensitive (gpd->l_job, !source);
+	gtk_widget_set_sensitive (gpd->l_paper, !source);
+	gtk_widget_set_sensitive (get_page (gpd->notebook, gpd->l_layout), !source);
+	gtk_widget_set_sensitive (get_page (gpd->notebook, gpd->l_job), !source);
+	gtk_widget_set_sensitive (get_page (gpd->notebook, gpd->l_paper), !source);
+}
+
+static void
+gnome_print_dialog_check_source (GnomePrintDialog *gpd)
+{
+	gchar *source;
+
+	source = (gchar *) gnome_print_config_get (gpd->config,
+		(const guchar *) "Settings.Document.Source");
+	gnome_print_dialog_set_has_source (gpd, source && strcmp (source, ""));
+	if (source)
+		g_free (source);
+}
+
+static void
+gnome_print_dialog_save_filter (GnomePrintDialog *gpd)
+{
+	gchar *d = NULL;
 
 	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
 
-	if (!gpd->config) {
-		GtkWidget *l;
+	if (gpd->filter)
+		d = gnome_print_filter_description (gpd->filter);
+	if (gpd->node_filter)
+		g_signal_handler_block (G_OBJECT (gpd->node_filter), gpd->signal_filter);
+	gnome_print_config_set (gpd->config,
+			(const guchar *) "Settings.Document.Filter",
+			(const guchar *) (d ? d : ""));
+	if (gpd->node_filter)
+		g_signal_handler_unblock (G_OBJECT (gpd->node_filter), gpd->signal_filter);
+	if (d)
+		g_free (d);
+}
 
-		gtk_container_remove (GTK_CONTAINER (GTK_DIALOG (gpd)->vbox),
-				gpd->notebook);
-		l = gtk_label_new (_("Error while loading printer configuration"));
-		gtk_widget_show (l);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), l, TRUE, TRUE, 0);
+static gboolean
+save_filter (gpointer data)
+{
+	GnomePrintDialog *gpd = GNOME_PRINT_DIALOG (data);
+
+	gnome_print_dialog_save_filter (gpd);
+	gpd->needs_save_filter = FALSE;
+	return FALSE;
+}
+
+static void
+gnome_print_dialog_schedule_save_filter (GnomePrintDialog *gpd)
+{
+	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
+
+	if (!gpd->needs_save_filter) {
+		gpd->needs_save_filter = TRUE;
+		g_idle_add (save_filter, gpd);
+	}
+}
+
+static void
+gnome_print_dialog_check_filter (GnomePrintDialog *gpd)
+{
+	gchar *d_new, *d_old = NULL;
+	GnomePrintFilter *f = NULL;
+	guint i;
+	struct {
+		GObject *o;
+		gboolean b;
+	} o[3];
+
+	d_new = (gchar *) gnome_print_config_get (gpd->config,
+			(const guchar *) "Settings.Document.Filter");
+	if (gpd->filter)
+		d_old = gnome_print_filter_description (gpd->filter);
+	if ((!d_new && !d_old) || (d_new && d_old && !strcmp (d_new, d_old))) {
+		if (d_new)
+			g_free (d_new);
+		if (d_old)
+			g_free (d_old);
 		return;
 	}
+	if (d_old)
+		g_free (d_old);
+	if (gpd->filter) {
+		g_object_unref (G_OBJECT (gpd->filter));
+		gpd->filter = NULL;
+	}
+	if (d_new)
+		gpd->filter = gnome_print_filter_new_from_description (d_new, NULL);
+
+	o[0].o = G_OBJECT (gpd->s_page);
+	o[1].o = G_OBJECT (gpd->s_copies);
+	o[2].o = G_OBJECT (gpd->s_layout);
+	o[0].b = FALSE;
+	o[1].b = FALSE;
+	o[2].b = FALSE;
+	for (f = gpd->filter; f; ) {
+		guint n = gnome_print_filter_count_successors (f);
+
+		for (i = 0; i < G_N_ELEMENTS (o); i++) {
+			GnomePrintFilter *fn = NULL;
+
+			if (o[i].b)
+				break;
+			g_object_set (o[i].o, "filter", f, NULL);
+			g_object_get (o[i].o, "filter", &fn, NULL);
+			o[i].b = f == fn;
+		}
+		if (!n || n > 1)
+			break;
+		f = gnome_print_filter_get_successor (f, 0);
+	}
+	for (i = 0; i < G_N_ELEMENTS (o); i++) {
+		GnomePrintFilter *fn = NULL;
+
+		if (!o[i].b) {
+			GParamSpec *ps = g_object_class_find_property (
+					G_OBJECT_GET_CLASS (o[i].o), "filter");
+			GValue v = {0,};
+
+			g_value_init (&v, G_PARAM_SPEC_VALUE_TYPE (ps));
+			g_param_value_set_default (ps, &v);
+			g_object_set_property (o[i].o, "filter", &v);
+			fn = g_value_get_object (&v);
+			g_object_ref (G_OBJECT (fn));
+			g_value_unset (&v);
+			if (gpd->filter) {
+				gnome_print_filter_append_predecessor (gpd->filter, fn);
+				g_object_unref (G_OBJECT (gpd->filter));
+			}
+			gpd->filter = fn;
+		}
+		g_object_get (o[i].o, "filter", &fn, NULL);
+	}
+	gnome_print_dialog_schedule_save_filter (gpd);
+	if (gpd->filter)
+		gnome_print_dialog_watch_filter (gpd, gpd->filter);
+}
+
+static void
+on_node_source_modified (GPANode *node, guint flags, GnomePrintDialog *gpd)
+{
+	gnome_print_dialog_check_source (gpd);
+}
+
+static void
+on_filter_notify (GObject *object, GParamSpec *pspec, GnomePrintDialog *gpd)
+{
+	gnome_print_dialog_schedule_save_filter (gpd);
+
+	if (!strcmp (pspec->name, "filters")) {
+		GValueArray *va = NULL;
+		guint i;
+
+		g_object_get (object, "filters", &va, NULL);
+		for (i = 0; i < va->n_values; i++)
+			gnome_print_dialog_watch_filter (gpd, GNOME_PRINT_FILTER (
+						g_value_get_object (g_value_array_get_nth (va, i))));
+	}
+}
+
+static void
+gnome_print_dialog_watch_filter (GnomePrintDialog *gpd, GnomePrintFilter *f)
+{
+	GClosure *closure;
+	guint i;
+
+	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
+	g_return_if_fail (GNOME_IS_PRINT_FILTER (f));
+
+	closure = g_cclosure_new (G_CALLBACK (on_filter_notify), gpd, NULL);
+	g_object_watch_closure (G_OBJECT (gpd), closure);
+	g_signal_connect_closure (G_OBJECT (f), "notify", closure, FALSE);
+	for (i = gnome_print_filter_count_filters (f); i > 0; i--)
+		gnome_print_dialog_watch_filter (gpd, gnome_print_filter_get_filter (f, i - 1));
+	for (i = gnome_print_filter_count_successors (f); i > 0; i--)
+		gnome_print_dialog_watch_filter (gpd, gnome_print_filter_get_successor (f, i - 1));
+}
+
+static void
+on_node_filter_modified (GPANode *node, guint flags, GnomePrintDialog *gpd)
+{
+	gnome_print_dialog_check_filter (gpd);
+}
+
+static void
+on_node_printer_modified (GPANode *node, guint flags, GnomePrintDialog *gpd)
+{
+	/* The filter gets reset in the config. We need to figure out why. */
+	gnome_print_dialog_schedule_save_filter (gpd);
+}
+
+static void
+gnome_print_dialog_set_config (GnomePrintDialog *gpd,
+		GnomePrintConfig *config)
+{
+	gint n, copies = 1;
+	gboolean collate = FALSE;
+	GtkWidget *hb, *l;
+
+	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
+	g_return_if_fail (!config || GNOME_IS_PRINT_CONFIG (config));
+
+	if (gpd->config == config)
+		return;
+
+	if (gpd->node_source) {
+		if (gpd->signal_source) {
+			g_signal_handler_disconnect (G_OBJECT (gpd->node_source),
+					gpd->signal_source);
+			gpd->signal_source = 0;
+		}
+		g_object_unref (G_OBJECT (gpd->node_source));
+		gpd->node_source = NULL;
+	}
+	if (gpd->node_filter) {
+		if (gpd->signal_filter) {
+			g_signal_handler_disconnect (G_OBJECT (gpd->node_filter),
+					gpd->signal_filter);
+			gpd->signal_filter = 0;
+		}
+		g_object_unref (G_OBJECT (gpd->node_filter));
+		gpd->node_filter = NULL;
+	}
+	if (gpd->node_printer) {
+		if (gpd->signal_printer) {
+			g_signal_handler_disconnect (G_OBJECT (gpd->node_printer),
+					gpd->signal_printer);
+			gpd->signal_printer = 0;
+		}
+		g_object_unref (G_OBJECT (gpd->node_printer));
+		gpd->node_printer = NULL;
+	}
+	if (gpd->config)
+		g_object_unref (G_OBJECT (gpd->config));
+	gpd->config = config;
+	if (config)
+		g_object_ref (G_OBJECT (config));
+
+	if (gpd->s_paper)
+		g_object_set (G_OBJECT (gpd->s_paper), "config", config, NULL);
+
+	if (!config)
+		return;
+
+	/* Copies & Collate */
+	gnome_print_config_get_int (gpd->config,
+		(const guchar *) GNOME_PRINT_KEY_NUM_COPIES, &copies);
+	gnome_print_config_get_boolean (gpd->config,
+		(const guchar *) GNOME_PRINT_KEY_COLLATE, &collate);
+	gnome_print_copies_selector_set_copies (
+		GNOME_PRINT_COPIES_SELECTOR (gpd->s_copies), copies, collate);
+	gnome_print_dialog_set_copies (gpd, copies, collate);
 
 	/* Add the printers page */
 	n = gtk_notebook_page_num (GTK_NOTEBOOK (gpd->notebook), gpd->printer);
@@ -164,77 +477,41 @@ gnome_print_dialog_load_config (GnomePrintDialog *gpd)
 	gtk_container_set_border_width (GTK_CONTAINER (hb), 4);
 	gtk_widget_show (gpd->printer);
 	gtk_box_pack_start (GTK_BOX (hb), gpd->printer, TRUE, TRUE, 0);
-
-	/* Add a paper page */
-	n = gtk_notebook_page_num (GTK_NOTEBOOK (gpd->notebook), gpd->s_paper);
-	if (n >= 0)
-		gtk_notebook_remove_page (GTK_NOTEBOOK (gpd->notebook), n);
-	gpd->s_paper = gnome_paper_selector_new (gpd->config);
-	gtk_container_set_border_width (GTK_CONTAINER (gpd->s_paper), 4);
-	gtk_widget_show (gpd->s_paper);
-	l = gtk_label_new_with_mnemonic (_("Paper"));
-	gtk_widget_show (l);
-	gtk_notebook_insert_page (GTK_NOTEBOOK (gpd->notebook), gpd->s_paper, l,
-			n < 0 ? gtk_notebook_get_n_pages (GTK_NOTEBOOK (gpd->notebook)) : n);
-
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (gpd->notebook),
 			gtk_notebook_page_num (GTK_NOTEBOOK (gpd->notebook), hb));
+
+	gnome_print_dialog_check_source (gpd);
+	gnome_print_dialog_check_filter (gpd);
+
+	/* Watch out for changes */
+	gpd->node_source = _gnome_print_config_ensure_key (config,
+			"Settings.Document.Source");
+	g_object_ref (G_OBJECT (gpd->node_source));
+	gpd->signal_source = g_signal_connect (G_OBJECT (gpd->node_source),
+			"modified", G_CALLBACK (on_node_source_modified), gpd);
+	gpd->node_filter = _gnome_print_config_ensure_key (config,
+			"Settings.Document.Source");
+	g_object_ref (G_OBJECT (gpd->node_filter));
+	gpd->signal_filter = g_signal_connect (G_OBJECT (gpd->node_filter),
+			"modified", G_CALLBACK (on_node_filter_modified), gpd);
+	gpd->node_printer = gpa_node_lookup (GNOME_PRINT_CONFIG_NODE (gpd->config),
+			(const guchar *) "Printer");
+	g_object_ref (G_OBJECT (gpd->node_printer));
+	gpd->signal_printer = g_signal_connect (G_OBJECT (gpd->node_printer),
+			"modified", G_CALLBACK (on_node_printer_modified), gpd);
 }
 
 static void
-gnome_print_dialog_update_content_info (GnomePrintDialog *gpd)
+gnome_print_dialog_finalize (GObject *object)
 {
-	guint c, n;
+	GnomePrintDialog *gpd = GNOME_PRINT_DIALOG (object);
 
-	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
+	/* Widgets got already destroyed */
+	gpd->s_paper = NULL;
 
-	if (!gpd->s_content)
-		return;
+	gnome_print_dialog_set_config (gpd, NULL);
 
-	g_object_get (G_OBJECT (gpd->s_content), "total", &n, "current", &c, NULL);
-	g_object_set (G_OBJECT (gpd->s_page), "total", n, "current", c, NULL);
-}
-
-static void
-on_notify (GObject *object, GParamSpec *pspec, GnomePrintDialog *gpd)
-{
-	gnome_print_dialog_update_content_info (gpd);
-}
-
-static void
-gnome_print_dialog_load_filter (GnomePrintDialog *gpd)
-{
-	GtkTextBuffer *buffer;
-	GtkTextIter start, end;
-	gchar *filter = NULL, *expert;
-
-	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
-
-	buffer = GTK_TEXT_VIEW (gpd->expert)->buffer;
-	gtk_text_buffer_get_start_iter (buffer, &start);
-	gtk_text_buffer_get_end_iter (buffer, &end);
-	expert = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-
-	if (gpd->config)
-		filter = (gchar *) gnome_print_config_get (gpd->config,
-				(const guchar *) "Settings.Output.Job.Filter");
-	if (!filter) {
-		if (strcmp ("", expert))
-			gtk_text_buffer_set_text (buffer, "", -1);
-		g_free (expert);
-		return;
-	}
-
-	if (strcmp (filter, expert))
-		gtk_text_buffer_set_text (buffer, filter, -1);
-	g_free (filter);
-	g_free (expert);
-}
-
-static void
-on_filter_modified (GPANode *node, guint flags, GnomePrintDialog *gpd)
-{
-	gnome_print_dialog_load_filter (gpd);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -255,36 +532,7 @@ gnome_print_dialog_set_property (GObject *object, guint prop_id,
 				(gpd->flags & GNOME_PRINT_DIALOG_COPIES) > 0, NULL);
 		break;
 	case PROP_PRINT_CONFIG:
-		if (gpd->config)
-			g_object_unref (G_OBJECT (gpd->config));
-		gpd->config = g_value_get_object (value);
-		if (gpd->config)
-			g_object_ref (G_OBJECT (gpd->config));
-		else
-			gpd->config = gnome_print_config_default ();
-		if (gpd->config) {
-			int copies = 1;
-			gboolean collate = FALSE;
-			GPANode *node;
-
-			gnome_print_config_get_int (gpd->config,
-					(const guchar *) GNOME_PRINT_KEY_NUM_COPIES, &copies);
-			gnome_print_config_get_boolean (gpd->config,
-					(const guchar *) GNOME_PRINT_KEY_COLLATE, &collate);
-			gnome_print_copies_selector_set_copies (
-					GNOME_PRINT_COPIES_SELECTOR (gpd->s_copies), copies, collate);
-			gnome_print_dialog_set_copies (gpd, copies, collate);
-
-			node = gpa_node_get_child_from_path (
-					GNOME_PRINT_CONFIG_NODE (gpd->config),
-					(const guchar *) "Settings.Output.Job.Filter");
-			g_signal_connect (G_OBJECT (node), "modified",
-					G_CALLBACK (on_filter_modified), gpd);
-		}
-		g_object_set (G_OBJECT (gpd->s_page), "config", gpd->config, NULL);
-		gnome_print_dialog_update_content_info (gpd);
-		gnome_print_dialog_load_config (gpd);
-		gnome_print_dialog_load_filter (gpd);
+		gnome_print_dialog_set_config (gpd, g_value_get_object (value));
 		break;
 	case PROP_CONTENT_SELECTOR:
 		if (gpd->s_content)
@@ -292,15 +540,17 @@ gnome_print_dialog_set_property (GObject *object, guint prop_id,
 					gpd->s_content);
 		gpd->s_content = g_value_get_object (value);
 		if (gpd->s_content) {
-			guint current = 0;
+			GValue v = {0,};
 
 			gtk_widget_show (gpd->e_content);
-			gtk_container_add (GTK_CONTAINER (gpd->e_content),
-					gpd->s_content);
+			gtk_container_add (GTK_CONTAINER (gpd->e_content), gpd->s_content);
 			gtk_widget_show (gpd->s_content);
-			g_object_get (G_OBJECT (gpd->s_content), "current", &current, NULL);
-			g_object_set (G_OBJECT (gpd->s_page), "current", current, NULL);
-			g_signal_connect (gpd->s_content, "notify", G_CALLBACK (on_notify), gpd);
+			g_object_get_property (G_OBJECT (gpd->s_content), "current", &v);
+			g_object_set_property (G_OBJECT (gpd->s_page), "current", &v);
+			g_object_get_property (G_OBJECT (gpd->s_content), "total", &v);
+			g_object_set_property (G_OBJECT (gpd->s_page), "total", &v);
+			g_signal_connect (gpd->s_content, "notify",
+					G_CALLBACK (on_content_selector_notify), gpd);
 		} else
 			gtk_widget_hide (gpd->e_content);
 		break;
@@ -331,6 +581,12 @@ gnome_print_dialog_get_property (GObject *object, guint prop_id,
 	case PROP_CONTENT_SELECTOR:
 		g_value_set_object (value, gpd->s_content);
 		break;
+	case PROP_PAGE_SELECTOR:
+		g_value_set_object (value, gpd->s_page);
+		break;
+	case PROP_PRINT_CONFIG:
+		g_value_set_object (value, gpd->config);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	}
@@ -340,18 +596,17 @@ static void
 gnome_print_dialog_class_init (GnomePrintDialogClass *class)
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
-	GtkObjectClass *object_class = (GtkObjectClass *) class;
 
 	parent_class = gtk_type_class (GTK_TYPE_DIALOG);
 
-	object_class->destroy = gnome_print_dialog_destroy;
-
 	gobject_class->set_property = gnome_print_dialog_set_property;
 	gobject_class->get_property = gnome_print_dialog_get_property;
+	gobject_class->finalize     = gnome_print_dialog_finalize;
+
 	g_object_class_install_property (gobject_class, PROP_PRINT_CONFIG,
 			g_param_spec_object ("print_config", "Print Config",
 				"Printing Configuration to be used",
-				GNOME_TYPE_PRINT_CONFIG, G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
+				GNOME_TYPE_PRINT_CONFIG, G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_class, PROP_PRINTER_SELECTOR,
 			g_param_spec_object ("printer_selector", "Printer selector",
 				"Printer selector", GNOME_TYPE_PRINTER_SELECTOR, G_PARAM_READABLE));
@@ -362,6 +617,10 @@ gnome_print_dialog_class_init (GnomePrintDialogClass *class)
 			g_param_spec_object ("content_selector", "Content selector",
 				"Content selector", GNOME_TYPE_PRINT_CONTENT_SELECTOR,
 				G_PARAM_READWRITE));
+	g_object_class_install_property (gobject_class, PROP_PAGE_SELECTOR,
+			g_param_spec_object ("page_selector", "Page selector",
+				"Page selector", GNOME_TYPE_PRINT_PAGE_SELECTOR,
+				G_PARAM_READABLE));
 	g_object_class_install_property (gobject_class, PROP_TITLE,
 			g_param_spec_string ("title", "Title",
 				"Title", _("Gnome Print Dialog"), G_PARAM_READWRITE));
@@ -401,35 +660,24 @@ gnome_print_dialog_response_cb (GtkDialog *dialog, gint response_id,
 }
 
 static void
-on_filter_description_changed (GtkTextBuffer *buffer, GnomePrintDialog *gpd)
+on_page_selector_notify (GObject *object, GParamSpec *pspec,
+		GnomePrintDialog *gpd)
 {
-	GtkTextIter start, end;
-	gchar *filter;
+	GValue v = {0,};
 
-	if (!gpd->config) return;
-
-	gtk_text_buffer_get_start_iter (buffer, &start);
-	gtk_text_buffer_get_end_iter (buffer, &end);
-	filter = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-	if (!filter) {
-		gnome_print_config_set (GNOME_PRINT_CONFIG (gpd->config),
-				(const guchar *) "Settings.Output.Job.Filter",
-				(const guchar *) "");
-		return;
+	g_value_init (&v, pspec->value_type);
+	if (!strcmp (pspec->name, "total")) {
+		g_object_get_property (object, "total", &v);
+		g_object_set_property (G_OBJECT (gpd->s_layout), "total", &v);
 	}
-
-	gnome_print_config_set (GNOME_PRINT_CONFIG (gpd->config),
-			(const guchar *) "Settings.Output.Job.Filter",
-			(const guchar *) filter);
-	g_free (filter);
+	g_value_unset (&v);
 }
 
 static void
 gnome_print_dialog_init (GnomePrintDialog *gpd)
 {
-	GtkWidget *vb, *l, *b, *w;
+	GtkWidget *vb, *l, *b;
 	gchar *text;
-	GtkTextBuffer *tb;
 
 	/* Set up the dialog */
 	gtk_window_set_title (GTK_WINDOW (gpd), _("Gnome Print Dialog"));
@@ -447,8 +695,7 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 	g_signal_connect (gpd, "response",
 		G_CALLBACK (gnome_print_dialog_response_cb), gpd);
 
-	gpd->notebook = gtk_notebook_new ();
-	gtk_container_set_border_width (GTK_CONTAINER (gpd->notebook), 4);
+	gpd->notebook = g_object_new (GTK_TYPE_NOTEBOOK, "border-width", 4, NULL);
 	gtk_widget_show (gpd->notebook);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (gpd)->vbox), gpd->notebook);
 
@@ -456,9 +703,10 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 	gpd->job = gtk_hbox_new (FALSE, 5);
 	gtk_widget_show (gpd->job);
 	gtk_container_set_border_width (GTK_CONTAINER (gpd->job), 4);
-	l = gtk_label_new_with_mnemonic (_("Job"));
-	gtk_widget_show (l);
-	gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->job, l);
+	gpd->l_job = gtk_label_new_with_mnemonic (_("Job"));
+	gtk_widget_show (gpd->l_job);
+	gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->job,
+			gpd->l_job);
 	vb = gtk_vbox_new (FALSE, PAD);
 	gtk_widget_show (vb);
 	gtk_box_pack_start (GTK_BOX (gpd->job), vb, FALSE, FALSE, 0);
@@ -485,6 +733,7 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 	gpd->s_page = g_object_new (GNOME_TYPE_PRINT_PAGE_SELECTOR, NULL);
 	gtk_widget_show (gpd->s_page);
 	gtk_box_pack_start (GTK_BOX (vb), gpd->s_page, FALSE, FALSE, 0);
+	g_signal_connect (gpd->s_page, "notify", G_CALLBACK (on_page_selector_notify), gpd);
 
 	/* Copies */
 	gpd->s_copies = g_object_new (GNOME_TYPE_PRINT_COPIES_SELECTOR, NULL);
@@ -494,20 +743,25 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 	gtk_box_pack_start (GTK_BOX (vb), gpd->s_copies, FALSE, FALSE, 0);
 	g_object_set_data (G_OBJECT (gpd->job), "copies", gpd->s_copies);
 
-	/* Expert mode */
-	w = g_object_new (GTK_TYPE_EXPANDER, "label", _("Expert mode"), 
-			"use-markup", TRUE, NULL);
-	/* gtk_widget_show (w); */
-	gtk_box_pack_start (GTK_BOX (gpd->job), w, FALSE, FALSE, 0);
-	tb = g_object_new (GTK_TYPE_TEXT_BUFFER, NULL);
-	gpd->expert = g_object_new (GTK_TYPE_TEXT_VIEW,
-			"wrap-mode", GTK_WRAP_WORD, NULL);
-	gtk_text_view_set_buffer (GTK_TEXT_VIEW (gpd->expert), tb);
-	g_signal_connect (G_OBJECT (tb), "changed",
-			      G_CALLBACK (on_filter_description_changed), gpd);
-	g_object_unref (G_OBJECT (tb));
-	gtk_widget_show (gpd->expert);
-	gtk_container_add (GTK_CONTAINER (w), gpd->expert);
+	/* Paper page */
+	gpd->s_paper = g_object_new (GNOME_TYPE_PAPER_SELECTOR, NULL);
+	gtk_container_set_border_width (GTK_CONTAINER (gpd->s_paper), 4);
+	gtk_widget_show (gpd->s_paper);
+	gpd->l_paper = gtk_label_new_with_mnemonic (_("Paper"));
+	gtk_widget_show (gpd->l_paper);
+	gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->s_paper,
+			gpd->l_paper);
+	g_signal_connect (G_OBJECT (gpd->s_paper), "notify",
+			G_CALLBACK (on_paper_selector_notify), gpd);
+
+	/* Layout page */
+	gpd->l_layout = gtk_label_new_with_mnemonic (_("Layout"));
+	gtk_widget_show (gpd->l_layout);
+	gpd->s_layout = g_object_new (GNOME_TYPE_PRINT_LAYOUT_SELECTOR,
+			"border-width", 4, NULL);
+	gtk_widget_show (gpd->s_layout);
+	gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->s_layout,
+			gpd->l_layout);
 }
 
 static void
@@ -604,9 +858,9 @@ gnome_print_dialog_new (GnomePrintJob *gpj, const guchar *title, gint flags)
 	config = gnome_print_job_get_config (gpj);
 	if (!config)
 		config = gnome_print_config_default ();
-
 	gpd = g_object_new (GNOME_TYPE_PRINT_DIALOG, "print-config", config,
 			"title", title, "flags", flags, NULL);
+	g_object_unref (G_OBJECT (config));
 
 	return GTK_WIDGET (gpd);
 }
