@@ -25,14 +25,18 @@
 
 #include "config.h"
 
-#include <popt.h>
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
 #include <stdlib.h>
 
 #include <gtk/gtk.h>
+
+#include <test-common.h>
+
 #include <libgnomeprint/gnome-print-config.h>
+
+#include <libgnomeprintui/gnome-print-job-preview.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <libgnomeprintui/gnome-printer-selector.h>
 #include <libgnomeprintui/gpaui/gpa-tree-viewer.h>
@@ -41,9 +45,8 @@
 
 #define MAGIC_NUMBER -100
 
-gboolean option_list_tests;
+gboolean option_list_tests = FALSE;
 gint     num_test = MAGIC_NUMBER;
-gchar *  num_test_str = NULL;
 gboolean debug = FALSE;
 gboolean tree = FALSE;
 gint     callback_count = 0;
@@ -56,18 +59,15 @@ typedef enum {
 	TEST_RETVAL_SUCCESS_LAST = 99,
 } TestRetval;
 
-poptContext popt;
-
-static struct poptOption options[] = {
-	{ "num", '\0', POPT_ARG_STRING, &num_test_str,   0,
+static const GOptionEntry options[] = {
+	{ "num", '\0', 0, G_OPTION_ARG_INT, &num_test,
 	  "Test number to run", "#"},
-	{ "list-tests", '\0', POPT_ARG_NONE, &option_list_tests,   0,
+	{ "list-tests", '\0', 0, G_OPTION_ARG_NONE, &option_list_tests,
 	  "List on the console the description of the test cases", NULL},
-	{ "tree",     '\0', POPT_ARG_NONE, &tree,   0,
+	{ "tree",     '\0', 0, G_OPTION_ARG_NONE, &tree,
 	  "Show the tree of GPANodes",  NULL},
-	{ "debug",    '\0', POPT_ARG_NONE, &debug, 0,
+	{ "debug",    '\0', 0, G_OPTION_ARG_NONE, &debug,
 	  "Print debugging output",          NULL},
-	POPT_AUTOHELP
 	{ NULL }
 };
 
@@ -174,7 +174,7 @@ test_dialog_show_all_pages (GnomePrintConfig *config)
 	
 	job = gnome_print_job_new (config);
 	dialog = gnome_print_dialog_new (job, (const guchar *) "Sample Print Dialog",
-					 GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
+			GNOME_PRINT_DIALOG_COPIES);
 	gtk_widget_show (dialog);
 	test_pos_window (dialog);
 	if (test_dialog_get_num_pages (dialog) != 3) {
@@ -184,7 +184,7 @@ test_dialog_show_all_pages (GnomePrintConfig *config)
 
 	job = gnome_print_job_new (config);
 	dialog = gnome_print_dialog_new (job, (const guchar *) "Sample Print Dialog",
-					 GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
+					 GNOME_PRINT_DIALOG_COPIES);
 	gtk_widget_show (dialog);
 	test_dialog_show_page (dialog, 1);
 	test_pos_window (dialog);
@@ -195,7 +195,7 @@ test_dialog_show_all_pages (GnomePrintConfig *config)
 
 	job = gnome_print_job_new (config);
 	dialog = gnome_print_dialog_new (job, (const guchar *) "Sample Print Dialog",
-					 GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
+					 GNOME_PRINT_DIALOG_COPIES);
 	gtk_widget_show (dialog);
 	test_pos_window (dialog);
 	test_dialog_show_page (dialog, 2);	
@@ -798,7 +798,7 @@ test_print_multiple (void)
 }
 
 static TestRetval
-test_dispatch (gint num_test)
+test_dispatch (void)
 {
 	TestRetval ret;
 
@@ -845,37 +845,41 @@ usage (gchar *error)
 {
 	g_print ("Error: %s\n\n", error);
 	g_print ("Usage: test-dialog --num=[num_test] [--tree]\n\n");
-	poptPrintHelp (popt, stdout, FALSE);
-	exit (TEST_RETVAL_BAD_PARAMETERS);
+	g_print ("Run '%s --help' to see a full list of available command line options.\n", g_get_prgname());
 	exit (TEST_RETVAL_BAD_PARAMETERS);
 }
 
 static void
-parse_command_line (int argc, const char ** argv, gint *num_test)
+parse_command_line (gint *argc, gchar *** argv)
 {
+	GOptionContext *goption_context;
+	GError *error = NULL;
 	gchar *env_test;
 	
-	popt = poptGetContext ("test_gpa", argc, argv, options, 0);
-	poptGetNextOpt (popt);
+	goption_context = g_option_context_new (NULL);
+	g_option_context_add_main_entries (goption_context, options, NULL);
+	g_option_context_parse (goption_context, argc, argv, &error);
+	g_option_context_free (goption_context);
+
+	if (error != NULL)
+		usage(error->message);
 
 	if (option_list_tests)
 		return;
 	
-	if (!num_test_str || num_test_str[0] == '\0') {
+	if (num_test == MAGIC_NUMBER) {
 		if (tree)
 			return;
 		env_test = getenv ("TEST_NUM");
 		if (env_test && env_test[0]) {
 			g_print ("TEST_NUM env variable = %s\n", env_test);
-			num_test_str = g_strdup (env_test);
+			num_test = atoi (env_test);
 		} else {
 			return;
 		}
 	} 
 
-	*num_test = atoi (num_test_str);
-
-	if (*num_test == -1) {
+	if (num_test == -1) {
 		GList *list = NULL;
 		/* We crash, this is part of the sanity check that allows us verify that crashes
 		 * are treated as errors when they happen when running ./run-tests
@@ -885,7 +889,7 @@ parse_command_line (int argc, const char ** argv, gint *num_test)
 		list->next = NULL;
 	}
 	
-	if (*num_test < 0 || (*num_test > max_num_test)) {
+	if (num_test < 0 || (num_test > max_num_test)) {
 		gchar *error;
 		error = g_strdup_printf ("num_test number out of range. Valid range is -1 to %d",
 					 max_num_test);
@@ -893,7 +897,7 @@ parse_command_line (int argc, const char ** argv, gint *num_test)
 	}
 	
 	if (debug)
-		g_print ("num_test is %d\n", *num_test);
+		g_print ("num_test is %d\n", num_test);
 }
 
 static void
@@ -914,15 +918,31 @@ handle_sigsegv (int i)
 	exit (TEST_RETVAL_CRASH);
 }
 
+static void
+on_dialog_response (GtkDialog *dialog, gint response, GnomePrintJob *job)
+{
+	GtkWidget *d;
+
+	switch (response) {
+	case 2:
+		d = gnome_print_job_preview_new (job, (const guchar *) "Test");
+		gtk_widget_show (d);
+		break;
+	default:
+		g_message ("Response %i.", response);
+		break;
+	}
+}
+
 int
-main (int argc, const char * argv[])
+main (int argc, char * argv[])
 {
 	TestRetval ret = TEST_RETVAL_SUCCESS;
 	struct sigaction sig;
 
 	gtk_init (&argc, (char ***) &argv);
 
-	parse_command_line (argc, argv, &num_test);
+	parse_command_line (&argc, &argv);
 
 	/* Catch sigsegv signals */
 	memset (&sig, 0, sizeof (sig));
@@ -940,7 +960,7 @@ main (int argc, const char * argv[])
 	}
 
 	if (num_test != MAGIC_NUMBER)
-		ret = test_dispatch (num_test);
+		ret = test_dispatch ();
 	else {
 		GnomePrintConfig *config;
 		GnomePrintJob *job;
@@ -950,17 +970,20 @@ main (int argc, const char * argv[])
 		config = gnome_print_job_get_config (job);
 		if (TRUE) {
 			GnomePrintContext *pc = NULL;
+			guint i;
 
 			g_object_get (G_OBJECT (job), "context", &pc, NULL);
-			my_draw (pc);
-			my_draw (pc);
+			for (i = 0; i < 5; i++)
+				test_print_page (pc, i);
+			gnome_print_job_close (job);
 			dialog = gnome_print_dialog_new (job, (const guchar *) "Test",
-					GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
-			gnome_print_dialog_construct_range_any (GNOME_PRINT_DIALOG (dialog),
-					GNOME_PRINT_RANGE_ALL |
-					GNOME_PRINT_RANGE_SELECTION, NULL, NULL, NULL);
-			gtk_dialog_run (GTK_DIALOG (dialog));
-			gtk_widget_destroy (dialog);
+					GNOME_PRINT_DIALOG_COPIES);
+			gtk_widget_show (dialog);
+			g_signal_connect (G_OBJECT (dialog), "delete_event",
+					G_CALLBACK (gtk_main_quit), NULL);
+			g_signal_connect (G_OBJECT (dialog), "response",
+					G_CALLBACK (on_dialog_response), job);
+			gtk_main ();
 		} else {
 			dialog = gnome_print_dialog_new (job, (const guchar *) "Test dialog 1", 0);
 			gtk_widget_show (dialog);
@@ -981,4 +1004,3 @@ main (int argc, const char * argv[])
 		
 	return ret;
 }
-
